"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
exports.__esModule = true;
var core_1 = require("@angular/core");
var forms_1 = require("@angular/forms");
var animations_1 = require("@angular/animations");
var LivesearchComponent = /** @class */ (function () {
    function LivesearchComponent(requestService, router, elRef) {
        this.requestService = requestService;
        this.router = router;
        this.elRef = elRef;
        this.onSelect = new core_1.EventEmitter();
        this.searchResult = [];
        this.loading = false;
        this.visible = true;
        this.searchInput = new forms_1.FormControl('');
        this.defaultSearchOptions = {
            searchParam: 'name',
            interval: 400,
            limit: 10,
            seeAllUrl: null,
            seeAllParams: {},
            seeAllPassSearchValue: true
        };
        this.defaultTextOptions = {
            seeAll: 'See all',
            noResults: 'No results',
            placeholder: 'Search'
        };
    }
    LivesearchComponent.prototype.doumentClicked = function (target) {
        if (!this.elRef.nativeElement.contains(target)) {
            this.visible = false;
        }
    };
    LivesearchComponent.prototype.ngOnInit = function () {
        var _this = this;
        this.searchOptions = Object.assign(this.defaultSearchOptions, this.searchOptions);
        this.textOptions = Object.assign(this.defaultTextOptions, this.textOptions);
        this.loadingSubscription = this.requestService.requestStartObs.subscribe(function () { return _this.loading = true; });
        this.init();
    };
    LivesearchComponent.prototype.init = function () {
        this.configureSearchService();
    };
    LivesearchComponent.prototype.getSearchParams = function () {
        if (this.searchOptions.seeAllPassSearchValue) {
            var key = this.searchOptions.searchParam;
            this.searchOptions.seeAllParams[key] = this.searchInput.value;
        }
        return this.searchOptions.seeAllParams;
    };
    LivesearchComponent.prototype.keyPressedOnSearchInput = function (event) {
        if (event.keyCode != 40 || !this.searchResult.length)
            return;
        var firstSearchItem = document.querySelector('.firstSearchResult');
        firstSearchItem.focus();
        event.preventDefault();
    };
    LivesearchComponent.prototype.keyPressedOnSearchResult = function (event) {
        var keycode = event.keyCode;
        if ([38, 40].indexOf(keycode) == -1)
            return;
        var target = event.currentTarget;
        var next = (keycode == 38 ? target.previousElementSibling : target.nextElementSibling);
        if (next && next.tagName == 'DIV') {
            next.focus();
            event.preventDefault();
        }
    };
    LivesearchComponent.prototype.configureSearchService = function () {
        this.requestService.timeToWait = this.searchOptions.interval;
        this.requestService.limit = this.searchOptions.limit;
        if (this.searchUrl) {
            this.requestService.searchUrl = this.searchUrl;
            this.requestService.searchParam = this.searchOptions.searchParam;
            this.requestService.search(this.searchInput.valueChanges)
                .subscribe(this.searchFinished.bind(this));
        }
        else {
            this.searchInput.valueChanges.subscribe(this.localSourceHandler.bind(this));
        }
    };
    LivesearchComponent.prototype.searchResultSelected = function (selectedItem) {
        this.onSelect.emit(selectedItem);
    };
    LivesearchComponent.prototype.localSourceHandler = function (query) {
        if (!query)
            return this.searchFinished([]);
        var retVal = this.localSource.filter(function (data) { return data.indexOf(query) != -1; });
        this.searchFinished(retVal);
    };
    LivesearchComponent.prototype.searchFinished = function (results) {
        this.loading = false;
        this.visible = true;
        this.searchResult = results.slice(0, this.searchOptions.limit);
        this.showEmptyMessage = results.length || !this.searchInput.value ? false : true;
    };
    LivesearchComponent.prototype.clearSearch = function () {
        this.searchInput.setValue('');
    };
    LivesearchComponent.prototype.inputFocused = function () {
        this.searchResult.length && (this.visible = true);
    };
    LivesearchComponent.prototype.navigateToSeeAll = function () {
        var queryParams = this.getSearchParams();
        this.router.navigate([this.searchOptions.seeAllUrl], { queryParams: queryParams });
    };
    LivesearchComponent.prototype.ngOnDestroy = function () {
        this.loadingSubscription.unsubscribe();
    };
    __decorate([
        core_1.Input()
    ], LivesearchComponent.prototype, "searchUrl");
    __decorate([
        core_1.Input()
    ], LivesearchComponent.prototype, "localSource");
    __decorate([
        core_1.ContentChild(core_1.TemplateRef)
    ], LivesearchComponent.prototype, "template");
    __decorate([
        core_1.Input()
    ], LivesearchComponent.prototype, "textOptions");
    __decorate([
        core_1.Input()
    ], LivesearchComponent.prototype, "searchOptions");
    __decorate([
        core_1.Output()
    ], LivesearchComponent.prototype, "onSelect");
    __decorate([
        core_1.HostListener('document:click', ['$event.target'])
    ], LivesearchComponent.prototype, "doumentClicked");
    LivesearchComponent = __decorate([
        core_1.Component({
            selector: 'livesearch',
            templateUrl: './livesearch.component.html',
            styleUrls: ['./livesearch.component.css'],
            animations: [
                animations_1.trigger('listAnimation', [
                    animations_1.transition('* => *', [
                        animations_1.query(':enter', animations_1.style({ opacity: 0 }), { optional: true }),
                        animations_1.query(':enter', animations_1.stagger('30ms', [
                            animations_1.animate('.2s ease-in', animations_1.keyframes([
                                animations_1.style({ opacity: 0, transform: 'translateY(-75%)', offset: 0 }),
                                animations_1.style({ opacity: 1, transform: 'translateY(0)', offset: 1.0 }),
                            ]))
                        ]), { optional: true })
                    ])
                ])
            ]
        }),
        __param(1, core_1.Optional())
    ], LivesearchComponent);
    return LivesearchComponent;
}());
exports.LivesearchComponent = LivesearchComponent;
